@@ -139,6 +139,7 @@ int nua_stack_set_defaults(nua_handle_t *nh,
   NHP_SET(nhp, only183_100rel, 0);
   NHP_SET(nhp, auto_answer, 0);
   NHP_SET(nhp, auto_ack, 1);
+  NHP_SET(nhp, timer_autorequire, 1);
   NHP_SET(nhp, invite_timeout, 120);
 
   nhp->nhp_session_timer = 1800;
@@ -156,13 +157,11 @@ int nua_stack_set_defaults(nua_handle_t *nh,
   NHP_SET(nhp, callee_caps, 0);
   NHP_SET(nhp, service_route_enable, 1);
   NHP_SET(nhp, path_enable, 1);
+  NHP_SET(nhp, retry_after_enable, 1);
 
   NHP_SET(nhp, refer_expires, 300);
   NHP_SET(nhp, refer_with_id, 1);
 
-  NHP_SET(nhp, auto302, 1);
-  NHP_SET(nhp, auto305, 1);
-
   NHP_SET(nhp, substate, nua_substate_active);
   NHP_SET(nhp, sub_expires, 3600);
 
@@ -174,6 +173,8 @@ int nua_stack_set_defaults(nua_handle_t *nh,
 
   NHP_SET(nhp, keepalive, 120000);
 
+  NHP_SET(nhp, auto_invite_100, 1);
+
   NHP_SET(nhp, appl_method,
 	  sip_allow_make(home, "INVITE, REGISTER, PUBLISH, SUBSCRIBE"));
 
@@ -273,6 +274,7 @@ int nua_stack_init_instance(nua_handle_t *nh, tagi_t const *tags)
  *   NUTAG_AUTOACK() \n
  *   NUTAG_AUTOALERT() \n
  *   NUTAG_AUTOANSWER() \n
+ *   NUTAG_CALL_TLS_ORQ_CONNECT_TIMEOUT() \n
  *   NUTAG_CALLEE_CAPS() \n
  *   NUTAG_DETECT_NETWORK_UPDATES() \n
  *   NUTAG_EARLY_ANSWER() \n
@@ -297,6 +299,7 @@ int nua_stack_init_instance(nua_handle_t *nh, tagi_t const *tags)
  *   NUTAG_ONLY183_100REL() \n
  *   NUTAG_OUTBOUND() \n
  *   NUTAG_PATH_ENABLE() \n
+ *   NUTAG_RETRY_AFTER_ENABLE() \n
  *   NUTAG_PROXY() (aka NTATAG_DEFAULT_PROXY()) \n
  *   NUTAG_REFER_EXPIRES() \n
  *   NUTAG_REFER_WITH_ID() \n
@@ -396,6 +399,7 @@ int nua_stack_init_instance(nua_handle_t *nh, tagi_t const *tags)
  *   NUTAG_AUTOACK() \n
  *   NUTAG_AUTOALERT() \n
  *   NUTAG_AUTOANSWER() \n
+ *   NUTAG_CALL_TLS_ORQ_CONNECT_TIMEOUT() \n
  *   NUTAG_CALLEE_CAPS() \n
  *   NUTAG_EARLY_ANSWER() \n
  *   NUTAG_EARLY_MEDIA() \n
@@ -419,6 +423,7 @@ int nua_stack_init_instance(nua_handle_t *nh, tagi_t const *tags)
  *   NUTAG_ONLY183_100REL() \n
  *   NUTAG_OUTBOUND() \n
  *   NUTAG_PATH_ENABLE() \n
+ *   NUTAG_RETRY_AFTER_ENABLE() \n
  *   NUTAG_PROXY() (aka NTATAG_DEFAULT_PROXY()) \n
  *   NUTAG_REFER_EXPIRES() \n
  *   NUTAG_REFER_WITH_ID() \n
@@ -704,6 +709,9 @@ static int nhp_set_tags(su_home_t *home,
     else if (tag == nutag_retry_count) {
       NHP_SET(nhp, retry_count, (unsigned)value);
     }
+    else if (tag == nutag_call_tls_orq_connect_timeout) {
+      NHP_SET(nhp, call_tls_orq_connect_timeout, (uint32_t)value);
+    }
     /* NUTAG_MAX_SUBSCRIPTIONS(max_subscriptions) */
     else if (tag == nutag_max_subscriptions) {
       NHP_SET(nhp, max_subscriptions, (unsigned)value);
@@ -744,13 +752,9 @@ static int nhp_set_tags(su_home_t *home,
     else if (tag == nutag_autoack) {
       NHP_SET(nhp, auto_ack, value != 0);
     }
-    /* NUTAG_AUTO302(auto302) */
-    else if (tag == nutag_auto302) {
-      NHP_SET(nhp, auto302, value != 0);
-    }
-    /* NUTAG_AUTO305(auto305) */
-    else if (tag == nutag_auto305) {
-      NHP_SET(nhp, auto305, value != 0);
+    /* NUTAG_TIMER_AUTOREQUIRE(timer_autorequire) */
+    else if (tag == nutag_timer_autorequire) {
+      NHP_SET(nhp, timer_autorequire, value != 0);
     }
     /* NUTAG_INVITE_TIMER(invite_timeout) */
     else if (tag == nutag_invite_timer) {
@@ -807,6 +811,10 @@ static int nhp_set_tags(su_home_t *home,
     else if (tag == nutag_path_enable) {
       NHP_SET(nhp, path_enable, value != 0);
     }
+    /* NUTAG_RETRY_AFTER_ENABLE(retry_after_enable) */
+    else if (tag == nutag_retry_after_enable) {
+      NHP_SET(nhp, retry_after_enable, value != 0);
+    }
     /* NUTAG_AUTH_CACHE(auth_cache) */
     else if (tag == nutag_auth_cache) {
       if (value >= 0 && value < (tag_value_t)_nua_auth_cache_invalid)
@@ -968,6 +976,16 @@ static int nhp_set_tags(su_home_t *home,
 	value = 0;
       NHP_SET_STR(nhp, organization, value);
     }
+    /* SIPTAG_VIA(via) */
+    else if (tag == siptag_via) {
+      NHP_SET_STR_BY_HEADER(nhp, via, value);
+    }
+    /* SIPTAG_VIA_STR(via_str) */
+    else if (tag == siptag_via_str) {
+      if (value == -1)
+	value = 0;
+      NHP_SET_STR(nhp, via, value);
+    }
     /* NUTAG_REGISTRAR(registrar) */
     else if (tag == nutag_registrar) {
       NHP_SET_STR_BY_URL(nhp, char, registrar, value);
@@ -1002,6 +1020,10 @@ static int nhp_set_tags(su_home_t *home,
     else if (tag == ntatag_default_proxy) {
       NHP_SET_STR_BY_URL(nhp, url_string_t, proxy, value);
     }
+    /* NUTAG_AUTO_INVITE_100() */
+    else if (tag == nutag_auto_invite_100) {
+      NHP_SET(nhp, auto_invite_100, value != 0);
+    }
     /* NUTAG_DETECT_NETWORK_UPDATES(detect_network_updates) */
     else if (ngp && tag == nutag_detect_network_updates) {
       int detector = (int)value;
@@ -1105,7 +1127,7 @@ int nhp_save_params(nua_handle_t *nh,
   su_home_t *home = nh->nh_home;
   nua_t *nua = nh->nh_nua;
   nua_handle_t *dnh = nua->nua_dhandle;
-  nua_handle_preferences_t *dst = nh->nh_prefs, old[1];
+  nua_handle_preferences_t *dst, old[1];
 
   if (gsrc) {
     *nua->nua_prefs = *gsrc;	/* No pointers this far */
@@ -1145,6 +1167,7 @@ int nhp_save_params(nua_handle_t *nh,
   NHP_ZAP_OVERRIDEN(old, dst, msg_header_free, allow_events);
   NHP_ZAP_OVERRIDEN(old, dst, su_free, user_agent);
   NHP_ZAP_OVERRIDEN(old, dst, su_free, organization);
+  NHP_ZAP_OVERRIDEN(old, dst, su_free, via);
   NHP_ZAP_OVERRIDEN(old, dst, su_free, m_display);
   NHP_ZAP_OVERRIDEN(old, dst, su_free, m_username);
   NHP_ZAP_OVERRIDEN(old, dst, su_free, m_params);
@@ -1236,6 +1259,14 @@ int nua_handle_save_tags(nua_handle_t *nh, tagi_t *tags)
     else if (t->t_tag == nutag_sips_url) {
       url = (url_string_t *)t->t_value;
     }
+    /* NUTAG_WS_URL_REF(url) */
+    else if (t->t_tag == nutag_ws_url) {
+      url = (url_string_t *)t->t_value;
+    }
+    /* NUTAG_WSS_URL_REF(url) */
+    else if (t->t_tag == nutag_wss_url) {
+      url = (url_string_t *)t->t_value;
+    }
   }
 
   su_home_auto(tmphome, sizeof tmphome);
@@ -1302,6 +1333,8 @@ static int nua_handle_param_filter(tagi_t const *f, tagi_t const *t)
 
   if (t->t_tag == nutag_url ||
       t->t_tag == nutag_sips_url ||
+      t->t_tag == nutag_ws_url ||
+      t->t_tag == nutag_wss_url ||
       t->t_tag == nutag_identity)
     return 0;
 
@@ -1465,6 +1498,7 @@ int nua_stack_set_smime_params(nua_t *nua, tagi_t const *tags)
  *   NUTAG_AUTOACK() \n
  *   NUTAG_AUTOALERT() \n
  *   NUTAG_AUTOANSWER() \n
+ *   NUTAG_CALL_TLS_ORQ_CONNECT_TIMEOUT() \n
  *   NUTAG_CALLEE_CAPS() \n
  *   NUTAG_DETECT_NETWORK_UPDATES() \n
  *   NUTAG_EARLY_ANSWER() \n
@@ -1489,6 +1523,7 @@ int nua_stack_set_smime_params(nua_t *nua, tagi_t const *tags)
  *   NUTAG_ONLY183_100REL() \n
  *   NUTAG_OUTBOUND() \n
  *   NUTAG_PATH_ENABLE() \n
+ *   NUTAG_RETRY_AFTER_ENABLE() \n
  *   NUTAG_REFER_EXPIRES() \n
  *   NUTAG_REFER_WITH_ID() \n
  *   NUTAG_REFRESH_WITHOUT_SDP() \n
@@ -1636,6 +1671,7 @@ int nua_stack_get_params(nua_t *nua, nua_handle_t *nh, nua_event_t e,
 			    : NULL)),
 
      TIF(NUTAG_RETRY_COUNT, retry_count),
+     TIF(NUTAG_CALL_TLS_ORQ_CONNECT_TIMEOUT, call_tls_orq_connect_timeout),
      TIF(NUTAG_MAX_SUBSCRIPTIONS, max_subscriptions),
 
      TIF(NUTAG_SOA_NAME, soa_name),
@@ -1647,6 +1683,7 @@ int nua_stack_get_params(nua_t *nua, nua_handle_t *nh, nua_event_t e,
      TIF(NUTAG_ONLY183_100REL, only183_100rel),
      TIF(NUTAG_AUTOANSWER, auto_answer),
      TIF(NUTAG_AUTOACK, auto_ack),
+     TIF(NUTAG_TIMER_AUTOREQUIRE, timer_autorequire),
      TIF(NUTAG_INVITE_TIMER, invite_timeout),
 
      TIFD(NUTAG_SESSION_TIMER, session_timer),
@@ -1663,13 +1700,11 @@ int nua_stack_get_params(nua_t *nua, nua_handle_t *nh, nua_event_t e,
      TIF(NUTAG_MEDIA_FEATURES, media_features),
      TIF(NUTAG_SERVICE_ROUTE_ENABLE, service_route_enable),
      TIF(NUTAG_PATH_ENABLE, path_enable),
+     TIF(NUTAG_RETRY_AFTER_ENABLE, retry_after_enable),
      TIF(NUTAG_AUTH_CACHE, auth_cache),
      TIF(NUTAG_REFER_EXPIRES, refer_expires),
      TIF(NUTAG_REFER_WITH_ID, refer_with_id),
 
-     TIF(NUTAG_AUTO302, auto302),
-     TIF(NUTAG_AUTO305, auto305),
-
      TIF(NUTAG_SUBSTATE, substate),
      TIF(NUTAG_SUB_EXPIRES, sub_expires),
 
@@ -1687,6 +1722,9 @@ int nua_stack_get_params(nua_t *nua, nua_handle_t *nh, nua_event_t e,
      TIF_SIP(SIPTAG_ORGANIZATION, organization),
      TIF(SIPTAG_ORGANIZATION_STR, organization),
 
+     TIF_SIP(SIPTAG_VIA, via),
+     TIF(SIPTAG_VIA_STR, via),
+
      TIF(NUTAG_INITIAL_ROUTE, initial_route),
      TIF_STR(NUTAG_INITIAL_ROUTE_STR, initial_route),
 
