@@ -192,7 +192,7 @@ int nua_stack_init(su_root_t *root, nua_t *nua)
       dnh->nh_ds->ds_leg == NULL ||
       nta_agent_set_params(nua->nua_nta, NTATAG_UA(1), TAG_END()) < 0 ||
       nua_stack_init_transport(nua, nua->nua_args) < 0) {
-    SU_DEBUG_1(("nua: initializing SIP stack failed\n"));
+    SU_DEBUG_1(("nua: initializing SIP stack failed\n" VA_NONE));
     return -1;
   }
 
@@ -313,7 +313,7 @@ int nua_stack_event(nua_t *nua, nua_handle_t *nh, msg_t *msg,
       assert(t == t_end); assert(b == end); (void)end;
     }
     else
-      p = e + 1;
+      p = ee + 1;
 
     ee->ee_nua = nua_stack_ref(nua);
     e->e_event = event;
@@ -396,7 +396,9 @@ void nua_application_event(nua_t *dummy, su_msg_r sumsg, nua_ee_data_t *ee)
 		      e->e_msg ? sip_object(e->e_msg) : NULL,
 		      e->e_tags);
 
-    su_msg_destroy(frame->nf_saved);
+	if (su_msg_is_non_null(frame->nf_saved)) {
+		su_msg_destroy(frame->nf_saved);
+	}
     nua->nua_current = frame->nf_next;
   }
 
@@ -417,6 +419,23 @@ msg_t *nua_current_request(nua_t const *nua)
   return NULL;
 }
 
+
+su_msg_t *nua_current_msg(nua_t const *nua, int clear)
+{
+	if (nua && nua->nua_current && su_msg_is_non_null(nua->nua_current->nf_saved)) {
+		su_msg_t *r = nua->nua_current->nf_saved[0];
+		if (clear) {
+			nua->nua_current->nf_saved[0] = NULL;
+		}
+		return r;
+		//return su_msg_data(nua->nua_current->nf_saved)->ee_data->e_msg;
+		
+	}
+
+  return NULL;
+}
+
+
 /** Get request message from saved nua event. @NEW_1_12_4.
  *
  * @sa nua_save_event(), nua_respond(), NUTAG_WITH_SAVED(),
@@ -638,16 +657,26 @@ void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_ee_data_t *ee)
     nua_stack_respond(nua, nh, e->e_status, e->e_phrase, tags);
     break;
   case nua_r_destroy:
-    nua_stack_destroy_handle(nua, nh, tags);
-    su_msg_destroy(nua->nua_signal);
+	  if (nh && !nh->nh_destroyed) {
+		  nua_stack_destroy_handle(nua, nh, tags);
+		  su_msg_destroy(nua->nua_signal);
+	  }
     return;
+  case nua_r_unref:
+    nua_unref(nua);
+    break;
+  case nua_r_handle_unref:
+    nua_handle_unref(nh);
+    break;
   default:
     break;
   }
 
   if (error < 0) {
-    nua_stack_event(nh->nh_nua, nh, NULL, event,
+    if (nh) {
+      nua_stack_event(nh->nh_nua, nh, NULL, event,
 		    NUA_ERROR_AT(__FILE__, __LINE__), NULL);
+    }
   }
 
   su_msg_destroy(nua->nua_signal);
@@ -856,7 +885,7 @@ void nua_stack_shutdown(nua_t *nua)
   if (status >= 200) {
     for (nh = nua->nua_handles; nh; nh = nh_next) {
       nh_next = nh->nh_next;
-      while (nh->nh_ds && nh->nh_ds->ds_usage) {
+      while (nh->nh_ds->ds_usage) {
 	nua_dialog_usage_remove(nh, nh->nh_ds, nh->nh_ds->ds_usage, NULL, NULL);
       }
     }
@@ -912,6 +941,10 @@ nua_handle_t *nh_validate(nua_t *nua, nua_handle_t *maybe)
 
 void nua_stack_destroy_handle(nua_t *nua, nua_handle_t *nh, tagi_t const *tags)
 {
+  if (nh->nh_destroyed) {
+	  return;
+  }
+
   if (nh->nh_notifier)
     nua_stack_terminate(nua, nh, (enum nua_event_e)0, NULL);
 
@@ -949,6 +982,12 @@ void nh_destroy(nua_t *nua, nua_handle_t *nh)
 {
   assert(nh); assert(nh != nua->nua_dhandle);
 
+  if (nh->nh_destroyed) {
+	  return;
+  }
+
+  nh->nh_destroyed = 1;
+
   if (nh->nh_notifier)
     nea_server_destroy(nh->nh_notifier), nh->nh_notifier = NULL;
 
@@ -1065,7 +1104,7 @@ sip_replaces_t *nua_stack_handle_make_replaces(nua_handle_t *nh,
 					       su_home_t *home,
 					       int early_only)
 {
-  if (nh && nh->nh_ds && nh->nh_ds->ds_leg)
+  if (nh && nh->nh_ds->ds_leg)
     return nta_leg_make_replaces(nh->nh_ds->ds_leg, home, early_only);
   else
     return NULL;
