@@ -98,7 +98,7 @@ struct sdp_session_s
   sdp_media_t       *sdp_media;         /**< Media descriptors */
 };
 
-/** Session description identification */ 
+/** Session description identification */
 struct sdp_origin_s
 {
   int               o_size;		/**< sizeof sdp_origin_t */
@@ -143,6 +143,7 @@ typedef enum
   sdp_bw_x,				/**< Unknown bandwidth type */
   sdp_bw_ct,				/**< Conference total */
   sdp_bw_as,				/**< Application-specific */
+  sdp_bw_tias,				/**< Application-specific */
 } sdp_bandwidth_e;
 
 /** Session or media bandwidth. */
@@ -150,7 +151,7 @@ struct sdp_bandwidth_s
 {
   int              b_size;		/**< Size fo sdp_bandwidth_t */
   sdp_bandwidth_t *b_next;		/**< Next bw description in list */
-  sdp_bandwidth_e  b_modifier;		/**< Meaning of value 
+  sdp_bandwidth_e  b_modifier;		/**< Meaning of value
 					     (total, or per application).  */
   sdp_text_t      *b_modifier_name;     /**< Modifier if not well-known */
   unsigned long    b_value;		/**< Bandwidth in kilobits per second */
@@ -197,7 +198,7 @@ typedef enum {
   sdp_key_clear,			/**< Key is included untransformed */
   sdp_key_base64,			/**< Key is encoded with base64 */
   sdp_key_uri,				/**< URI used to obtain a key */
-  sdp_key_prompt			/**< No key is included, 
+  sdp_key_prompt			/**< No key is included,
 					     prompt user for key */
 } sdp_key_method_e;
 
@@ -229,8 +230,10 @@ typedef enum
   sdp_media_data,			/**< Bulk data transfer */
   sdp_media_control,			/**< Additional conference control */
   sdp_media_message,			/**< Messaging sessions*/
-  sdp_media_image,			/**< Image browsing sessions for JPIP, and T.38 */
-  sdp_media_red				/**< Redundancy. @NEW_1_12_4. */
+  sdp_media_image,			/**< Image browsing sessions,
+					 *   e.g., JPIP or T.38. */
+  sdp_media_red,			/**< Redundancy. @NEW_1_12_4. */
+  sdp_media_text,			/**< Realtime Text */
 } sdp_media_e;
 
 /** Media transport protocol. */
@@ -242,25 +245,29 @@ typedef enum
   sdp_proto_rtp = 256,			/**< RTP/AVP */
   sdp_proto_srtp = 257,			/**< RTP/SAVP  */
   sdp_proto_udptl = 258,		/**< UDPTL. @NEW_1_12_4. */
+  sdp_proto_msrp =  259,		/**< TCP/MSRP @NEW_MSRP*/
+  sdp_proto_msrps = 260,		/**< TCP/TLS/MSRP @NEW_MSRP*/
+  sdp_proto_extended_srtp = 261, /** WEBRTC SAVPF */
+  sdp_proto_extended_rtp = 262, /** WEBRTC AVPF */
   sdp_proto_tls = 511,			/**< TLS over TCP */
   sdp_proto_any = 512		        /**< * wildcard */
 } sdp_proto_e;
 
 /** Session mode. @note Identical to rtp_mode_t. */
 typedef enum {
-  sdp_inactive = 0, 
-  sdp_sendonly = 1, 
-  sdp_recvonly = 2, 
+  sdp_inactive = 0,
+  sdp_sendonly = 1,
+  sdp_recvonly = 2,
   sdp_sendrecv = sdp_sendonly | sdp_recvonly
 } sdp_mode_t;
 
 /** Media announcement.
- * 
+ *
  * This structure describes one media type, e.g., audio.  The description
  * contains the transport address (IP address and port) used for the group,
  * the transport protocol used, the media formats or RTP payload types, and
  * optionally media-specific bandwidth specification, encryption key and
- * attributes. 
+ * attributes.
  *
  * There is a pointer (m_user) for the application data, too.
  */
@@ -285,9 +292,9 @@ struct sdp_media_s
   sdp_attribute_t  *m_attributes;	/**< Media attributes */
 
   void             *m_user;	        /**< User data. */
-  
+
   /** Rejected media */
-  unsigned          m_rejected : 1;     
+  unsigned          m_rejected : 1;
   /** Inactive, recvonly, sendonly, sendrecv */
   /* sdp_mode_t */ unsigned m_mode : 2;
   unsigned          : 0;
@@ -302,7 +309,7 @@ struct sdp_list_s
 };
 
 /** Mapping from RTP payload to codec.
- * 
+ *
  * The sdp_rtpmap_t() structure defines a mapping from an RTP payload to a
  * particular codec.  In case of well-known payloads, the sdp_rtpmap_t()
  * structure may be predefined, that is, generated by SDP parser without
@@ -334,31 +341,31 @@ sdp_origin_t    *sdp_origin_dup(su_home_t *, sdp_origin_t const *);
 
 /** Duplicate an SDP connection structure. */
 SOFIAPUBFUN
-sdp_connection_t *sdp_connection_dup(su_home_t *home, sdp_connection_t const *);
+sdp_connection_t *sdp_connection_dup(su_home_t *, sdp_connection_t const *);
 
 /** Duplicate an SDP bandwidth structure. */
 SOFIAPUBFUN
-sdp_bandwidth_t  *sdp_bandwidth_dup(su_home_t *home, sdp_bandwidth_t const *);
+sdp_bandwidth_t  *sdp_bandwidth_dup(su_home_t *, sdp_bandwidth_t const *);
 
 /** Duplicate an SDP time structure. */
 SOFIAPUBFUN
-sdp_time_t       *sdp_time_dup(su_home_t *home, sdp_time_t const *);
+sdp_time_t       *sdp_time_dup(su_home_t *, sdp_time_t const *);
 
 /** Duplicate an SDP repeat structure. */
 SOFIAPUBFUN
-sdp_repeat_t     *sdp_repeat_dup(su_home_t *home, sdp_repeat_t const *);
+sdp_repeat_t     *sdp_repeat_dup(su_home_t *, sdp_repeat_t const *);
 
 /** Duplicate an SDP timezone structure. */
 SOFIAPUBFUN
-sdp_zone_t       *sdp_zone_dup(su_home_t *home, sdp_zone_t const *);
+sdp_zone_t       *sdp_zone_dup(su_home_t *, sdp_zone_t const *);
 
 /** Duplicate an SDP key structure. */
 SOFIAPUBFUN
-sdp_key_t        *sdp_key_dup(su_home_t *home, sdp_key_t const *);
+sdp_key_t        *sdp_key_dup(su_home_t *, sdp_key_t const *);
 
 /** Duplicate an SDP attribute structure. */
 SOFIAPUBFUN
-sdp_attribute_t  *sdp_attribute_dup(su_home_t *home, sdp_attribute_t const *);
+sdp_attribute_t  *sdp_attribute_dup(su_home_t *, sdp_attribute_t const *);
 
 /** Duplicate an SDP media description structure. */
 SOFIAPUBFUN
@@ -372,11 +379,11 @@ sdp_media_t *sdp_media_dup_all(su_home_t *, sdp_media_t const *,
 
 /** Duplicate a list structure. */
 SOFIAPUBFUN
-sdp_list_t       *sdp_list_dup(su_home_t *home, sdp_list_t const *);
+sdp_list_t       *sdp_list_dup(su_home_t *, sdp_list_t const *);
 
 /** Duplicate an rtpmap structure. */
 SOFIAPUBFUN
-sdp_rtpmap_t     *sdp_rtpmap_dup(su_home_t *home, sdp_rtpmap_t const *);
+sdp_rtpmap_t     *sdp_rtpmap_dup(su_home_t *, sdp_rtpmap_t const *);
 
 /** Compare two session descriptions. */
 SOFIAPUBFUN int sdp_session_cmp(sdp_session_t const *a,
@@ -436,8 +443,8 @@ SOFIAPUBFUN sdp_attribute_t  *sdp_attribute_find(sdp_attribute_t const *a,
 						 char const *name);
 
 /** Find named attribute from given lists. */
-SOFIAPUBFUN sdp_attribute_t *sdp_attribute_find2(sdp_attribute_t const *a, 
-						 sdp_attribute_t const *a2, 
+SOFIAPUBFUN sdp_attribute_t *sdp_attribute_find2(sdp_attribute_t const *a,
+						 sdp_attribute_t const *a2,
 						 char const *name);
 
 /** Get session mode from attribute list. */
@@ -445,26 +452,26 @@ SOFIAPUBFUN sdp_mode_t sdp_attribute_mode(sdp_attribute_t const *a,
 					  sdp_mode_t defmode);
 
 /** Get session mode from attribute list. */
-SOFIAPUBFUN sdp_attribute_t *sdp_attribute_by_mode(su_home_t *, 
+SOFIAPUBFUN sdp_attribute_t *sdp_attribute_by_mode(su_home_t *,
 						   sdp_mode_t mode);
 
 /** Find a mapped attribute. */
-SOFIAPUBFUN 
-sdp_attribute_t *sdp_attribute_mapped_find(sdp_attribute_t const *a, 
-					   char const *name, 
+SOFIAPUBFUN
+sdp_attribute_t *sdp_attribute_mapped_find(sdp_attribute_t const *a,
+					   char const *name,
 					   int pt, char **return_result);
 
 /** Append a attribute to a list of attributes. */
-SOFIAPUBFUN void sdp_attribute_append(sdp_attribute_t **list, 
-			  sdp_attribute_t const *a);
+SOFIAPUBFUN void sdp_attribute_append(sdp_attribute_t **list,
+				      sdp_attribute_t const *a);
 
 /** Replace a attribute within a list of attributes. */
-SOFIAPUBFUN int sdp_attribute_replace(sdp_attribute_t **list, 
+SOFIAPUBFUN int sdp_attribute_replace(sdp_attribute_t **list,
 				      sdp_attribute_t *a,
 				      sdp_attribute_t **return_replaced);
 
 /** Remove a named attribute from a list of attributes. */
-SOFIAPUBFUN sdp_attribute_t *sdp_attribute_remove(sdp_attribute_t **list, 
+SOFIAPUBFUN sdp_attribute_t *sdp_attribute_remove(sdp_attribute_t **list,
 						  char const *name);
 
 /* Return 1 if m= line struct matches with given type and name */
@@ -504,8 +511,8 @@ enum sdp_parse_flags_e {
   /** Accept only conforming SDP */
   sdp_f_strict = 1,
   /** Accept any network type. */
-  sdp_f_anynet = 2,		
-  /** Reallocate message. */ 
+  sdp_f_anynet = 2,
+  /** Reallocate message. */
   sdp_f_realloc = 4,
   /** Include well-known rtpmaps in message, too */
   sdp_f_all_rtpmaps = 8,
@@ -546,8 +553,9 @@ SOFIAPUBFUN su_home_t *sdp_parser_home(sdp_parser_t *);
 typedef struct sdp_printer_s sdp_printer_t;
 typedef sdp_printer_t *sdp_printer;
 
-SOFIAPUBFUN sdp_printer_t *sdp_print(su_home_t *, sdp_session_t const *session, 
-				     char msgbuf[], isize_t maxmsgsize, int flags);
+SOFIAPUBFUN sdp_printer_t *sdp_print(su_home_t *, sdp_session_t const *,
+				     char msgbuf[], isize_t maxmsgsize,
+				     int flags);
 SOFIAPUBFUN char const *sdp_printing_error(sdp_printer_t *p);
 SOFIAPUBFUN char const *sdp_message(sdp_printer_t *p);
 SOFIAPUBFUN isize_t sdp_message_size(sdp_printer_t *p);
