@@ -41,7 +41,7 @@
 #define MSG_HDR_T       union sip_header_u
 
 #include <sofia-sip/su_alloc.h>
-#include <sofia-sip/string0.h>
+#include <sofia-sip/su_string.h>
 
 #include "sofia-sip/sip_parser.h"
 #include <sofia-sip/sip_util.h>
@@ -940,7 +940,7 @@ static int sip_addr_update(msg_common_t *h,
   if (name == NULL) {
     a->a_tag = NULL;
   }
-  else if (namelen == strlen("tag") && !strncasecmp(name, "tag", namelen)) {
+  else if (namelen == strlen("tag") && su_casenmatch(name, "tag", namelen)) {
     a->a_tag = value;
   }
 
@@ -983,7 +983,7 @@ int sip_addr_tag(su_home_t *home, sip_addr_t *a, char const *tag)
       value = tag;
 
     if (a->a_tag) {
-      if (strcasecmp(a->a_tag, value) == 0)
+      if (su_casematch(a->a_tag, value))
 	return 0;
       else
 	return -1;
@@ -1287,7 +1287,7 @@ sip_cseq_t *sip_cseq_create(su_home_t *home,
   sip_cseq_t *cs;
 
   if (method)
-    method_name = sip_method_name(method, method_name);
+    method_name = sip_method_name((sip_method_t)method, method_name);
 
   if (method_name == NULL)
     return NULL;
@@ -1298,7 +1298,7 @@ sip_cseq_t *sip_cseq_create(su_home_t *home,
 
   if (cs) {
     cs->cs_seq = seq;
-    cs->cs_method = method;
+    cs->cs_method = (sip_method_t)method;
     if (!method)
       method_name = strcpy((char *)(cs + 1), method_name);
     cs->cs_method_name = method_name;
@@ -1391,18 +1391,23 @@ issize_t sip_contact_d(su_home_t *home,
 		       char *s,
 		       isize_t slen)
 {
-  sip_contact_t *m = (sip_contact_t *)h;
+  sip_contact_t *m;
 
   assert(h);
+  for(;;) {
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+	  m = (sip_contact_t *)h;
 
-  if (sip_name_addr_d(home, &s, &m->m_display, m->m_url,
-		      &m->m_params, &m->m_comment) == -1)
-    return -1;
+	  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+		  *s = '\0', s += span_lws(s + 1) + 1;
+	  
+	  if (sip_name_addr_d(home, &s, &m->m_display, m->m_url,
+						  &m->m_params, &m->m_comment) == -1)
+		  return -1;
+
+	  msg_parse_next_field_without_recursion();
+  }
 
-  return msg_parse_next_field(home, h, s, slen);
 }
 
 
@@ -1456,12 +1461,12 @@ static int sip_contact_update(msg_common_t *h,
     m->m_q = NULL;
     m->m_expires = NULL;
   }
-  else if (namelen == 1 && strncasecmp(name, "q", 1) == 0) {
+  else if (namelen == 1 && su_casenmatch(name, "q", 1)) {
     /* XXX - check for invalid value? */
     m->m_q = value;
   }
   else if (namelen == strlen("expires") &&
-	   !strncasecmp(name, "expires", namelen)) {
+	   su_casenmatch(name, "expires", namelen)) {
     m->m_expires = value;
   }
 
@@ -2086,7 +2091,7 @@ static int sip_retry_after_update(msg_common_t *h,
     af->af_duration = NULL;
   }
   else if (namelen == strlen("duration") &&
-	   !strncasecmp(name, "duration", namelen)) {
+	   su_casenmatch(name, "duration", namelen)) {
     af->af_duration = value;
   }
 
@@ -2106,18 +2111,24 @@ issize_t sip_any_route_d(su_home_t *home,
 			 char *s,
 			 isize_t slen)
 {
-  sip_route_t *r = (sip_route_t *)h;
+	sip_route_t *r;
+
 
   assert(h);
+  for (;;) {
+	  r = (sip_route_t *)h;
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+	  while (*s == ',')  { /* Ignore empty entries (comma-whitespace) */
+		  *s = '\0', s += span_lws(s + 1) + 1;
+	  }
 
-  if (sip_name_addr_d(home, &s, &r->r_display,
-		      r->r_url, &r->r_params, NULL) < 0)
-    return -1;
+	  if (sip_name_addr_d(home, &s, &r->r_display, r->r_url, &r->r_params, NULL) < 0) {
+		  return -1;
+	  }
+
+	  msg_parse_next_field_without_recursion();
+  }
 
-  return msg_parse_next_field(home, h, s, slen);
 }
 
 issize_t sip_any_route_e(char b[], isize_t bsiz, sip_header_t const *h, int flags)
@@ -2534,29 +2545,32 @@ SIP_HEADER_CLASS(via, "Via", "v", v_params, prepend, via);
 
 issize_t sip_via_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
-  sip_via_t *v = (sip_via_t *)h;
+  sip_via_t *v;
 
   assert(h);
+  for (;;) {
+	  v = (sip_via_t *)h;
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+	  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+		  *s = '\0', s += span_lws(s + 1) + 1;
 
-  /* sent-protocol sent-by *( ";" via-params ) [ comment ] */
+	  /* sent-protocol sent-by *( ";" via-params ) [ comment ] */
+	  
+	  /* Parse protocol */
+	  if (sip_transport_d(&s, &v->v_protocol) == -1)
+		  return -1;
+	  /* Host (and port) */
+	  if (msg_hostport_d(&s, &v->v_host, &v->v_port) == -1)
+		  return -1;
+	  /* Parameters */
+	  if (*s == ';' && msg_params_d(home, &s, &v->v_params) == -1)
+		  return -1;
+	  /* Comment */
+	  if (*s == '(' && msg_comment_d(&s, &v->v_comment) == -1)
+		  return -1;
 
-  /* Parse protocol */
-  if (sip_transport_d(&s, &v->v_protocol) == -1)
-    return -1;
-  /* Host (and port) */
-  if (msg_hostport_d(&s, &v->v_host, &v->v_port) == -1)
-    return -1;
-  /* Parameters */
-  if (*s == ';' && msg_params_d(home, &s, &v->v_params) == -1)
-    return -1;
-  /* Comment */
-  if (*s == '(' && msg_comment_d(&s, &v->v_comment) == -1)
-    return -1;
-
-  return msg_parse_next_field(home, h, s, slen);
+	  msg_parse_next_field_without_recursion();
+  }
 }
 
 issize_t sip_via_e(char b[], isize_t bsiz, sip_header_t const *h, int flags)
@@ -2635,7 +2649,7 @@ static int sip_via_update(msg_common_t *h,
     v->v_rport = NULL;
     v->v_comp = NULL;
   }
-#define MATCH(s) (namelen == strlen(#s) && !strncasecmp(name, #s, strlen(#s)))
+#define MATCH(s) (namelen == strlen(#s) && su_casenmatch(name, #s, strlen(#s)))
 
    else if (MATCH(ttl)) {
      v->v_ttl = value;
@@ -2761,7 +2775,7 @@ char const *sip_via_port(sip_via_t const *v, int *using_rport)
 
     if (v->v_rport && !v->v_maddr /* multicast */) {
       if (v->v_protocol == sip_transport_udp ||
-	  strcasecmp(v->v_protocol, sip_transport_udp) == 0)
+	  su_casematch(v->v_protocol, sip_transport_udp))
 	port = v->v_rport, *using_rport = 0;
       else if (*using_rport)
 	port = v->v_rport;
@@ -2783,3 +2797,118 @@ char const *sip_via_port(sip_via_t const *v, int *using_rport)
   else
     return SIP_DEFAULT_SERV;	/* 5060 */
 }
+
+/**@SIP_HEADER sip_identity Identity Header
+ *
+ * The Identity header field specifies the "logical" recipient of the
+ * request. It is defined in @RFC8224 with semantics shown below,
+ * though for now it's parsed to a single 'value' field.
+ *
+ * @code
+ *  Identity =	"Identity" HCOLON signed-identity-digest SEMI
+ *				ident-info *( SEMI ident-info-params )
+ *              signed-identity-digest = 1*(base64-char / ".")
+ *              ident-info = "info" EQUAL ident-info-uri
+ *              ident-info-uri = LAQUOT absoluteURI RAQUOT
+ *              ident-info-params = ident-info-alg / ident-type /
+ *              ident-info-extension
+ *              ident-info-alg = "alg" EQUAL token
+ *              ident-type = "ppt" EQUAL token
+ *              ident-info-extension = generic-param
+ *
+ *              base64-char = ALPHA / DIGIT / "/" / "+"
+ * @endcode
+ *
+ * The parsed Identity header is stored in #sip_identity_t structure.
+ */
+
+/**@ingroup sip_identity
+ * @typedef typedef struct sip_identity_s sip_identity_t;
+ *
+ * The structure #sip_identity_t contains representation of @Identity header.
+ *
+ * The #sip_identity_t is defined as follows:
+ * @code
+ * typedef struct {
+ *   sip_common_t       id_common[1];    // Common fragment info
+ *   sip_error_t		*id_next;        // Link to next (dummy)
+ *   char const			*id_value;		// Identity
+ *   char const			*id_info;		// Info param containing URL of the cert, with no '<','>'
+ * } sip_identity_t;
+ * @endcode
+ *
+ */
+
+static msg_xtra_f sip_identity_dup_xtra;
+static msg_dup_f sip_identity_dup_one;
+static msg_update_f sip_identity_update;
+
+msg_hclass_t sip_identity_class[] =
+SIP_HEADER_CLASS(identity, "Identity", "", id_common, single, identity);
+
+issize_t sip_identity_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
+{
+  sip_identity_t *id = (sip_identity_t *)h;
+  char const *p = NULL, *pp = NULL, *ppp = NULL, *ie = NULL;
+  char *result = NULL;
+  size_t len = 0;
+
+  id->id_value = strdup(s);
+  id->id_info = NULL;
+
+  p = strstr(s, "info=");
+  if (p) {
+
+	  ie = strchr(p, ';');
+	  pp = strchr(p, '<');
+	  ppp = strchr(p, '>');
+
+	  // allow for a spaces between "info=" and opening '<'
+	  // extract URI from inside "<>" but allow empty - let the higher level app decide what to do about it
+	  if (ie && pp && ppp && (pp < ppp) && (ppp < ie)) {
+
+		  len = ppp - pp;
+		  if ((result = malloc(len))) {
+			  memcpy(result, pp + 1, len - 1);
+			  result[len - 1] = '\0';
+			  id->id_info = result;
+		  }
+	  }
+  }
+
+  return 0;
+}
+
+issize_t sip_identity_e(char b[], isize_t bsiz, sip_header_t const *h, int flags)
+{
+  sip_identity_t const *id = (sip_identity_t *)h;
+
+  return snprintf(b, bsiz, "%s", id->id_value);
+}
+
+isize_t sip_identity_dup_xtra(sip_header_t const *h, isize_t offset)
+{
+  sip_identity_t const *id = (sip_identity_t *)h;
+  return offset + MSG_STRING_SIZE(id->id_value);
+}
+
+char *sip_identity_dup_one(sip_header_t *dst, sip_header_t const *src,
+		       char *b, isize_t xtra)
+{
+  sip_identity_t *id = (sip_identity_t *)dst;
+  sip_identity_t const *o = (sip_identity_t *)src;
+
+  MSG_STRING_DUP(b, id->id_value, o->id_value);
+
+  return b;
+}
+
+static int sip_identity_update(msg_common_t *h,
+			     char const *name, isize_t namelen,
+			     char const *value)
+{
+  sip_identity_t *id = (sip_identity_t *)h;
+
+  id->id_value = strdup(value);
+  return 0;
+}
