@@ -24,7 +24,7 @@
 
 /**@ingroup sip_status_codes
  * @CFILE sip_status.c
- * 
+ *
  * SIP status codes and standard phrases.
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>.
@@ -37,26 +37,27 @@
 #include <stddef.h>
 #include <sofia-sip/sip_status.h>
 
-char const 
+char const
   sip_100_Trying[] =                   "Trying",
   sip_180_Ringing[] =                  "Ringing",
   sip_181_Call_is_being_forwarded[] =  "Call Is Being Forwarded",
   sip_182_Queued[] =                   "Queued",
   sip_183_Session_progress[] =         "Session Progress",
-  
+
   sip_200_OK[] =                       "OK",
   sip_202_Accepted[] =                 "Accepted",
-  
+
   sip_300_Multiple_choices[] =         "Multiple Choices",
   sip_301_Moved_permanently[] =        "Moved Permanently",
   sip_302_Moved_temporarily[] =        "Moved Temporarily",
   sip_305_Use_proxy[] =                "Use Proxy",
   sip_380_Alternative_service[] =      "Alternative Service",
-  
+
   sip_400_Bad_request[] =              "Bad Request",
   sip_401_Unauthorized[] =             "Unauthorized",
   sip_402_Payment_required[] =         "Payment Required",
   sip_403_Forbidden[] =                "Forbidden",
+  sip_403_Stale_date[] =               "Stale Date",
   sip_404_Not_found[] =                "Not Found",
   sip_405_Method_not_allowed[] =       "Method Not Allowed",
   sip_406_Not_acceptable[] =           "Not Acceptable",
@@ -75,7 +76,13 @@ char const
   sip_421_Extension_required[] =       "Extension Required",
   sip_422_Session_timer[] =            "Session Interval Too Small",
   sip_423_Interval_too_brief[] =       "Interval Too Brief",
-  
+
+  sip_428_Use_identity_header[] =      "Use Identity Header",
+  sip_436_Bad_identity_info[] =        "Bad Identity Info",
+  sip_437_Unsupported_credential[] =   "Unsupported Credential",
+  sip_438_Invalid_identity_header[] =  "Invalid Identity Header",
+  sip_439_Stale_date[] =               "Stale Date",
+
   sip_480_Temporarily_unavailable[] =  "Temporarily Unavailable",
   sip_481_No_transaction[] =           "Call/Transaction Does Not Exist",
   sip_482_Loop_detected[] =            "Loop Detected",
@@ -104,6 +111,7 @@ char const
   sip_603_Decline[] =                  "Decline",
   sip_604_Does_not_exist_anywhere[] =  "Does Not Exist Anywhere",
   sip_606_Not_acceptable[] =           "Not Acceptable",
+  sip_607_Unwanted[] =                 "Unwanted",
   sip_687_Dialog_terminated[] =        "Dialog Terminated"
   ;
 
@@ -114,13 +122,13 @@ char const
  * empty string "" is returned.
  *
  * @param status well-known status code in range 100..699
- * 
+ *
  * @return
  * A response message corresponding to status code, or NULL upon an error.
  */
 char const *sip_status_phrase(int status)
 {
-  if (status < 100 || status > 699) 
+  if (status < 100 || status > 699)
     return NULL;
 
   switch (status) {
@@ -191,6 +199,7 @@ char const *sip_status_phrase(int status)
   case 603: return sip_603_Decline;
   case 604: return sip_604_Does_not_exist_anywhere;
   case 606: return sip_606_Not_acceptable;
+  case 607: return sip_607_Unwanted;
   case 687: return sip_687_Dialog_terminated;
   }
 
