@@ -257,9 +257,11 @@ sip_contact_string_from_via(su_home_t *home,
 
   /* Make transport parameter lowercase */
   if (transport && strlen(transport) < (sizeof _transport)) {
-    char *s = strcpy(_transport, transport);
+    char *s;
     short c;
 
+    strcpy(_transport, transport);
+
     for (s = _transport; (c = *s) && c != ';'; s++)
       if (isupper(c))
 	*s = tolower(c);
@@ -394,7 +396,8 @@ issize_t sip_header_field_e(char *b, isize_t bsiz, sip_header_t const *h, int fl
 char *sip_header_as_string(su_home_t *home, sip_header_t const *h)
 {
   ssize_t len;
-  char *rv, s[128];
+  char *rv, s[256];
+  ssize_t n;
 
   if (h == NULL)
     return NULL;
@@ -412,7 +415,8 @@ char *sip_header_as_string(su_home_t *home, sip_header_t const *h)
   for (rv = su_alloc(home, len);
        rv;
        rv = su_realloc(home, rv, len)) {
-    ssize_t n = sip_header_field_e(rv, len, h, 0);
+	memset(rv,0,len);
+    n = sip_header_field_e(rv, len, h, 0);
     if (n > -1 && n + 1 <= len)
       break;
     if (n > -1)			/* glibc >2.1 */
@@ -668,12 +672,27 @@ sip_route_t *sip_route_fixdup(su_home_t *home, sip_route_t const *route)
   return sip_route_fixdup_as(home, sip_route_class, route);
 }
 
+static void sip_fragment_clear_chain(sip_header_t *h)
+{
+  void const *next;
+
+  for (; h; h = h->sh_succ) {
+    next = (char *)h->sh_data + h->sh_len;
+
+    sip_fragment_clear(h->sh_common);
+
+    if (!next ||
+	!h->sh_succ ||
+	h->sh_next != h->sh_succ ||
+	h->sh_succ->sh_data != next ||
+	h->sh_succ->sh_len)
+      return;
+  }
+}
+
 /**@ingroup sip_route
  *
  * Fix @Route header.
- *
- * If URI has no parameters and the header contains a single @c ;lr header
- * parameter, the "lr" is moved as URI parameter.
  */
 sip_route_t *sip_route_fix(sip_route_t *route)
 {
@@ -699,7 +718,7 @@ sip_route_t *sip_route_fix(sip_route_t *route)
       for (i = 0; r->r_params[i]; i++)
 	((char const **)r->r_params)[i] = r->r_params[i + 1];
 
-      msg_fragment_clear_chain((msg_header_t *)h);
+      sip_fragment_clear_chain(h);
     }
   }
 
@@ -717,9 +736,14 @@ sip_via_t *sip_via_remove(msg_t *msg, sip_t *sip)
   if (sip == NULL)
     return NULL;
 
-  v = sip->sip_via;
+  for (v = sip->sip_via; v; v = v->v_next) {
+    sip_fragment_clear(v->v_common);
 
-  if (v)
+    if (v->v_next != (void *)v->v_common->h_succ)
+      break;
+  }
+
+  if ((v = sip->sip_via))
     msg_header_remove(msg, (msg_pub_t *)sip, (msg_header_t *)v);
 
   return v;
@@ -1400,6 +1424,15 @@ int sip_response_terminates_dialog(int response_code,
     */
     *return_graceful_terminate_usage = 0;
     return 0;
+
+  case 607: /** @par 607 Unwanted
+
+      This response is similar to 603, except it informs the caller
+      that the call is unwanted and may be blacklisted.  Added by
+      RFC-8197
+    */
+    *return_graceful_terminate_usage = 0;
+    return 0;
   }
 
   /* Do not change graceful_terminate */
