@@ -152,7 +152,7 @@ _su_task_r su_task_init(su_task_r task)
 {
   assert(task);
 
-  memset(task, 0, sizeof(task));
+  memset(task, 0, sizeof(su_task_r));
   return task;
 }
 
@@ -422,7 +422,6 @@ su_root_clone_initializer(su_root_t *root,
  *
  * Allocate and initialize the instance of su_root_t.
  *
- * @param self      pointer to a root object.
  * @param magic     pointer to user data
  *
  * @return A pointer to allocated su_root_t instance, NULL on error.
@@ -714,7 +713,7 @@ int su_root_set_max_defer(su_root_t *self, su_duration_t max_defer)
  * activated, however, they are deferred no longer than the maximum defer
  * time. The maximum defer time is 15 seconds by default.
  *
- * @param self pointer to root object
+ * @param root pointer to root object
  *
  * @return Maximum defer time
  *
