@@ -185,7 +185,7 @@ int url_reserved_p(char const *s)
  * The number of characters in corresponding but escaped string.
  *
  * You can handle a part of URL with reserved characters like this:
- * @code
+ *@code
  * if (url_reserved_p(s))  {
  *   n = malloc(url_esclen(s, NULL) + 1);
  *   if (n) url_escape(n, s);
@@ -477,6 +477,7 @@ char const* url_scheme(enum url_type_e url_type)
   case url_cid:    return "cid";
   case url_msrp:   return "msrp";
   case url_msrps:  return "msrps";
+  case url_urn:    return "urn";
   case url_wv:     return "wv";
   default:
     assert(url_type == url_unknown);
@@ -535,6 +536,8 @@ enum url_type_e url_get_type(char const *scheme, size_t len)
     test_scheme(sip); test_scheme(sips); break;
   case 't': case 'T':
     test_scheme(tel); break;
+  case 'u': case 'U':
+    test_scheme(urn); break;
   case 'w': case 'W':
     test_scheme(wv); break;
 
@@ -567,7 +570,7 @@ static
 int _url_d(url_t *url, char *s)
 {
   size_t n, p;
-  char *s0, rest_c, *host, *user;
+  char rest_c, *host, *user;
   int have_authority = 1;
 
   memset(url, 0, sizeof(*url));
@@ -578,8 +581,6 @@ int _url_d(url_t *url, char *s)
     return 0;
   }
 
-  s0 = s;
-
   n = strcspn(s, ":/?#");
 
   if (n && s[n] == ':') {
@@ -1090,7 +1091,7 @@ char *copy(char *buf, char *end, char const *src)
  * @param buf     Buffer for non-constant strings copied from @a src.
  * @param bufsize Size of @a buf.
  * @param dst     Destination URL structure.
- * @param src     Source URL structure.
+ *@param src     Source URL structure.
  *
  * @return Number of characters required for
  * duplicating the strings in @a str, or -1 if an error
@@ -1174,7 +1175,7 @@ issize_t url_dup(char *buf, isize_t bufsize, url_t *dst, url_t const *src)
  * @param buf     Buffer for non-constant strings copied from @a src.
  * @param end     End of @a buf.
  * @param dst     Destination URL structure.
- * @param src     Source URL structure.
+ *@param src     Source URL structure.
  *
  * @return
  * The macro URL_DUP() returns pointer to first unused byte in the
@@ -1798,6 +1799,7 @@ char const *url_port_default(enum url_type_e url_type)
     return "9999";		/* XXXX */
 
   case url_tel:
+  case url_urn:
   case url_fax:
   case url_modem:
   case url_im:
@@ -1843,6 +1845,7 @@ char const *url_tport_default(enum url_type_e url_type)
   case url_im:
   case url_pres:
   case url_cid:
+  case url_urn:
   case url_wv:
 
   default:			/* Unknown scheme */
@@ -2110,33 +2113,30 @@ char *url_query_as_header_string(su_home_t *home,
   if (!s)
     return NULL;
 
-  for (i = 0, j = 0; query[i];) {
-    n = strcspn(query + i, "=");
-    if (!query[i + n])
+  for (i = 0, j = 0; s[i];) {
+    n = strcspn(s + i, "=");
+    if (!s[i + n])
       break;
-    if (n == 4 && su_strncasecmp(query + i, "body", 4) == 0) {
+    if (n == 4 && strncasecmp(s + i, "body", 4) == 0) {
       if (b_start)
 	break;
-      b_start = i + n + 1, b_len = strcspn(query + b_start, "&");
-      i = b_start + b_len;
-      if (!query[i])
-        break;
-      i++;
+      b_start = i + n + 1, b_len = strcspn(s + b_start, "&");
+      i = b_start + b_len + 1;
       continue;
     }
     if (i != j)
-      memcpy(s + j, query + i, n);
+      memmove(s + j, s + i, n);
     s[j + n] = ':';
     i += n + 1, j += n + 1;
-    n = strcspn(query + i, "&");
-    j += url_unescape_to(s + j, query + i, n);
+    n = strcspn(s + i, "&");
+    j += url_unescape_to(s + j, s + i, n);
     i += n;
-    if (query[i]) {
+    if (s[i]) {
       s[j++] = '\n', i++;
     }
   }
 
-  if (query[i])
+  if (s[i])
     return (void)su_free(home, s), NULL;
 
   if (b_start) {
